* ------------------------------------------------------------------- Smooth_x_y_data
      Program Smooth_x_y_data


* Computes the Fourier amplitude spectra for specified time series.  
* Writes output in smc or column format.

* The output file name has the same name as the input file, with the specified
* character string appended to the name.

* Input from a control file; here is an example:
*!Control file for program Smooth_x_y_data.for
*! Revision of program involving a change in the control file on this date:
*   02/29/12
*! As many comment lines as desired, each starting with "!"
*! The string "pp:" indicates a new set of processing parameters
*! to be applied to the following smc files.  The parameters are given on the
*! lines following "pp:", until the next "pp:" line or until "stop" is 
*! encountered.
*!
*! -----------------------------------------------------------------------------
*!
*! Meaning of smoothing input parameters
*!
*! NO SMOOTHING
*! itype = 0
*! SMOOTHING OVER EQUALLY SPACED FREQUENCIES
*! itype = 1: box weighting function
*!   smooth_param = width of box weighting function (x units)
*! itype = 2: triangular weighting function
*!   smooth_param = width of triangular weighting function (x units)
*! SMOOTHING OVER LOGARITHMICALLY SPACED FREQUENCIES
*! itype = 3: box weighting function
*!   smooth_param = xi, which is the fraction of a decade for the
*!                  box weighting function 
*! itype = 4: triangular weighting function
*!   smooth_param = xi, which is the fraction of a decade for the
*!                  triangular weighting function 
*! itype = 5: Konno and Ohmachi weighting function (see BSSA 88, 228-241)
*!   smooth_param = xi, which is the fraction of a decade for which
*!                  the Konno and Ohmachi weighting function is greater
*!                  than 0.043.(it is related to
*!                  their smoothing parameter b by the equation
*!                  b = 4.0/smooth_param, so we have this correspondence between
*!                  b and smooth_param
*!                         b smooth_param 
*!                        10         0.40
*!                        20         0.20
*!                        40         0.10
*!                  
*!                  b = 40 seems to be commonly used, but I do not think that it
*!                  gives enough smoothing; I PREFER SMOOTH_PARAM = 0.2, 
*!                  corresponding to b = 20. 
*!
*! ipow = power of y to be smoothed (2 = smoothing energy spectrum)
*!
*! dx_smooth: Note: need df_smooth for linearly-spaced smoothers. 
*! For general x data, it is
*! the spacing between x values, assumed to be constant.  If itype = 1 or 2,
*! dx_smooth is obtained as x_in(2)-x_in(1).  The reason for
*! including it as an input parameter is to "fool" the
*! program to do smoothing over a specified number of points by
*! setting dx_smooth = 1 and smooth_param = number of points (including 
*! points with zero weight at ends; e.g., smooth_param = 5 will 
*! give a smoother with weights 0, 1/4, 2/4, 1/4, 0; smooth_param
*! should be odd).
*!
*! Meaning of x output specification parameters:
*!
*!Specify x values for output (y/n):
*! <enter Y or N>
*!x output specification: 
*!  If specify_x_out = Y, then enter the 
*!    nx_out, x_out(1), x_out(2)..., x_out(nx_out)
*!  If specify_x_out = N, then enter 
*!    x_out_low, x_out_high, log-spaced (0=N, 1=Y), x_out_param
*!         if x_out_param = 0.0, there is no interpolation, and the xin values 
*!            are used between x_out_low and x_out_high (if want all xin values, set
*!            x_out_low and x_out_high to values outside the range of xin(low) and xin(high) 
*!              (log-spaced is ignored).
*!         if x_out_param /= 0.0 and log-spaced = 0, then x_out_param is the spacing of the
*!            interpolated x values between x_out_low and x_out_high
*!         if x_out_param /= 0.0 and log-spaced = 1, then x_out_param is the number of 
*!            interpolated x between x_out_low and x_out_high
*! -----------------------------------------------------------------------------
*!
*!Name of summary file:
* smooth_x_y_data.sum
*!A set of parameters and files:
*PP: new set of parameters
*!smoothing: itype, ipow, dx_smooth (not used if itype >= 3, otherwise x_in spacing), smooth_param
*! (see above for the meaning of these input parameters):
*   0 1 0.0 0.20
*!Specify x values for output (y/n)::
*  N
*!x output specification (if (specify_x_out) then nx_out, x_out(1)...x_out(nx_out) ELSE
*!            x_out_low, x_out_high, flag_for_x_out_log_spacing (0=N, 1=Y), x_out_param)                      
*   0.0 50.0 0 0.0 
*!character string to append to filename:
*   .smooth_ko_0.2.col
*!N lines to skip to get to data
* 1
*!N columns to read, columns for x and y columns 
*  3 1 3
*!Files to process:
* x_y_data.txt
*stop
 
* Dates: 02/29/12 - Written by D.M. Boore, patterned after SMC2FS2
 
c
c Dimension and declaration statements:

      real  x_out(:), y_out(:)
      allocatable :: x_out, y_out

      real x_in(100000), y_in(100000), dum(30)
      character f_in*200, f_out*200
      character xin_c(:)*11, yin_c(:)*11
      character x_out_c(:)*11, y_out_c(:)*11
      allocatable xin_c, yin_c, x_out_c, y_out_c
 
      real flag_for_x_out_log_spacing

      integer npts, nx_out, nc_f_in
      real x_out_low, x_out_high
      real dx_smooth, signnpw2

      character char_head(11)*80, comments(500)*80, cmnts2skip(500)*80
      real real_head(50)
      integer int_head(48)

      character buf*300, buf_upper*4, 
     :        f_sum*300, c2*2, c2up*2,
     :        ext_c*10, string*300, string4name*300,
     :        specify_x_out*10
     
      logical x_out_allocated_in_pp, reset_smc_out

      character f_ctl*60, date*8, 
     :        time_start*10, time_stop*10 

      character date_ctl_correct*8, date_ctl_in*30
      
      logical f_ctl_exist, f_in_exist, dc_remove, datetime_l
      logical log_spaced_x_out

      f_ctl_exist = .false.
      do while (.not. f_ctl_exist)
        f_ctl = ' '
        write(*, '(a\)') 
     :    ' Enter name of control file ("enter"=Smooth_x_y_data.ctl;'//
     :    ' ctl-brk to quit): '
        read(*, '(a)') f_ctl
        if (f_ctl(1:4) == '    ') f_ctl = 'Smooth_x_y_data.ctl'
        call trim_c(f_ctl, nc_f_ctl)
        inquire(file=f_ctl(1:nc_f_ctl), exist=f_ctl_exist)
        if (.not. f_ctl_exist) then
          write(*,'(a)') ' ******* FILE DOES NOT EXIST ******* '
        end if
      end do

      call get_lun(nu_ctl)
      open(unit=nu_ctl,file=f_ctl(1:nc_f_ctl),status='unknown')

      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)
      date_ctl_in = ' '
      read(nu_ctl,'(a)') date_ctl_in
      call trim_c(date_ctl_in,nc_date_ctl_in)
      
      date_ctl_correct = ' '
      date_ctl_correct = '02/29/12'
      call trim_c(date_ctl_correct,nc_date_ctl_correct)

      if (date_ctl_correct(1:nc_date_ctl_correct) /= 
     :    date_ctl_in(1:nc_date_ctl_in)) then
        write(*,'(a)') 
     :     ' The control file has the wrong date; update your '//
     :       'control file and rerun the program!'
        close(nu_ctl)
        stop
      end if
      
      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)

      f_sum = ' '
      read(nu_ctl,'(a)') f_sum
      call trim_c(f_sum,nc_f_sum)
      call get_lun(nu_sum)
      open(unit=nu_sum,file=f_sum(1:nc_f_sum),status='unknown')

      write(nu_sum,'(a)') ' Output of program Smooth_x_y_data:'
      date = ' '
      time_begin = ' '
* Standard Fortran 90 intrinsic Subroutine DATE_AND_TIME
      call DATE_AND_TIME( date, time_start )
* Date is returned as 'CCYYMMDD'
* Time is returned as 'hhmmss.sss'
      write(nu_sum, '(2a)') ' Date: '//date(1:4)//'/'//
     :                                 date(5:6)//'/'//date(7:8)
      write(nu_sum, '(2a)') ' Time: '//time_start(1:2)//':'//
     :           time_start(3:4)//':'//time_start(5:10)
      write(nu_sum,'(2a)') '   Control file = ', f_ctl(1:nc_f_ctl)

      x_out_allocated_in_pp = .false.

      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)

      loop_over_files: DO
        buf = ' '
        read(nu_ctl,'(a)',IOSTAT=iostatus) buf
        if (iostatus /= 0) EXIT
        call trim_c(buf,nc_buf)
        buf_upper = ' '
        buf_upper = buf(1:4)
        call upstr(buf_upper)
        if(nc_buf == 0 .or. buf(1:1) == '!') CYCLE
        if(buf_upper(1:4) .eq. 'STOP') EXIT
      
        processing_parameters: IF(buf_upper(1:3) == 'PP:') THEN  ! a new set of processing parameters
        
! x_out previously allocated in a specify-x_out set of processing
! parameters; deallocate for a new set of processing parameters.  Note 
! that when the x_out values are not specified, x_out is allocated and
! deallocated for each file.
          IF (x_out_allocated_in_pp) THEN 
            deallocate(x_out)
          END IF
            
! Initialize the input arrays for each new PP set:
          call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
          read(nu_ctl,*) itype, ipow, dx_smooth, smooth_param
          write(nu_sum,'(a)') 
     :     ' itype, ipow, dx_smooth_in, smooth_param ='
          write(nu_sum,*) 
     :     itype, ipow, dx_smooth, smooth_param

          call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
          specify_x_out = ' '
          read(nu_ctl,'(a)',IOSTAT=iostatus) specify_x_out
          if (iostatus /= 0) EXIT
          call trim_c(specify_x_out,nc_specify_x_out)
          call upstr(specify_x_out)

          IF (specify_x_out(1:1) == 'Y') THEN
          
            CALL skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
            READ (nu_ctl,*) nx_out

            ALLOCATE ( x_out(nx_out) )
            x_out_allocated_in_pp = .true.
            
            BACKSPACE nu_ctl   ! need to read nx_out twice, the first time for
                               ! allocation of x_out

            READ (nu_ctl,*) nx_out, (x_out(i), i = 1, nx_out)
            
            x_out_param = 999.0  ! if 0.0, subroutine smooth_interpolate assumes 
                              ! xin spacing
            x_out_low = 0.0
            x_out_high = 0.0
            log_spaced_x_out = .false.
            WRITE (nu_sum,'(a)') 
     :      ' nx_out, x_out = '
            WRITE (nu_sum, '(1x,i3, 8(1x,es10.3))') 
     :        nx_out, (x_out(i), i = 1, nx_out)
     
          ELSE
          
            CALL skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
            READ (nu_ctl,*) 
     :            x_out_low, x_out_high, flag_for_x_out_log_spacing, 
     :            x_out_param
            
            IF (x_out_param == 0.0) THEN  ! no interpolation
              log_spaced_x_out = .false.
            ELSE
              IF (flag_for_x_out_log_spacing == 0.0) THEN
                 log_spaced_x_out = .false.
              ELSE
                 log_spaced_x_out = .true.   ! note: dx_out_intrp is set to the number
                                         ! of x_out values later, in the 
                                         ! x_out allocation statements
              END IF
            END IF
            
            WRITE (nu_sum,'(a)') 
     :      ' x_out_low, x_out_high, log_spaced_x_out, x_out_param = '
            WRITE (nu_sum, '(1x, 2(1x,es10.3), 1x,l1, 1x,es10.3)') 
     :        x_out_low, x_out_high, log_spaced_x_out, x_out_param
     
          END IF

          call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
          string4name = ' '
          read(nu_ctl,'(a)') string4name
          call trim_c(string4name, nc_string4name)
          write(nu_sum,'(a)') 
     :   ' string4name = '//string4name(1:nc_string4name)

          call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
          read(nu_ctl,*) n_lines_to_skip
          write(*,'(a,1x, i3)') ' n_lines_to_skip = ', n_lines_to_skip

          call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
          read(nu_ctl,*) ncol2read, icolx, icoly
          write(*,'(a,3(1x,i3))') 
     :  ' ncol2read, icolx, icoly = ', ncol2read, icolx, icoly

          call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)

          CYCLE
          
        END IF processing_parameters 

      
        f_in = ' '
        f_in = buf(1:nc_buf)
        call trim_c(f_in, nc_f_in)
        write(nu_sum,'(a)') ' f_in: '//f_in(1:nc_f_in)
        write(nu_sum,'(a,1x,i3)') ' nc_f_in = ', nc_f_in 
        inquire(file=f_in(1:nc_f_in), exist=f_in_exist)
        if (.not. f_in_exist) then
            write(*,'(a)') ' ****** FILE '//f_in(1:nc_f_in)// 
     :          ' DOES NOT EXIST; SKIPPING!!! ******* '
          CYCLE
        end if
        call get_lun(nu_in)
        open (nu_in, file=f_in(1:nc_f_in), status='unknown')

* Passed tests; proceed

* Extract the time series from the input file:

        print *
        print *, 
     :   'Processing file:', f_in(1:nc_f_in)
        write(nu_sum,*)
        write(nu_sum,'(1x,a)') 
     :   'Processing file:', f_in(1:nc_f_in)

* Read x, y, values from the input file:

        nx_in = 0

        call skip(nu_in,n_lines_to_skip)  !   skip to data

        x_in = 0.0
        y_in = 0.0

        DO

          read(nu_in,*, iostat=iread_status) (dum(i), i=1, ncol2read)
          if(iread_status /= 0) then
            close(nu_in)
            EXIT
          end if

          nx_in = nx_in + 1    

          x_in(nx_in) = dum(icolx)
          y_in(nx_in) = dum(icoly)

        END DO
        
        write(*,'(a)') ' finished reading file '//
     :                 f_in(1:nc_f_in)
        close(nu_in)

!Smooth the values

        if (itype <= 2 .and. dx_smooth == 0.0) then
          dx_smooth = x_in(2) - x_in(1)
        end if
        write(nu_sum,'(a)') 
     :     ' itype, ipow, dx_smooth after reading data, smooth_param ='
          write(nu_sum,*) 
     :     itype, ipow, dx_smooth, smooth_param


! Fill the x_out array (it is used by the smooth_interpolate subroutine):

        IF (specify_x_out(1:1) == 'Y') THEN  ! If 'Y', then nx_out, x_out 
                                            ! have been specified by the calling program
          allocate ( y_out(nx_out) )
          dx_out = -1.0
          m_start = -1
          m_stop  = -1
          
        ELSE  
      
          IF (x_out_param == 0.0) THEN  ! use input x values, between x_out_low and x_out_high

            m_start = 1
            call locate(x_in, nx_in, x_out_low, m_start)
   
            call locate(x_in, nx_in, x_out_high, m_stop)
            
            IF (m_stop > nx_in) THEN
              m_stop = nx_in
            END IF
   
            nx_out = m_stop - m_start + 1
            allocate ( x_out(nx_out), y_out(nx_out) )
            DO j = 1, nx_out
              x_out(j) = x_in(j + m_start - 1)
            END DO
            dx_out = x_in(2) - x_in(1)  ! x might not be evenly spaced. Is this an assumption of the 
                                        ! of the smoothing operators?  This program was based on smc2fs2,
                                        ! for which the x values are evenly spaced, as they come from an FFT
        
          ELSE                       ! use interpolated x values
        
            IF (log_spaced_x_out) THEN
              nx_out = int(x_out_param)
              allocate ( x_out(nx_out), y_out(nx_out) )
              dlogx = alog10(x_out_high/x_out_low)/
     :              real(nx_out-1)
              DO j = 1, nx_out
                x_out(j) = x_out_low*10.0**(float(j-1) * dlogx)
              END DO 
              dx_out = -1.0
              m_start = -1
              m_stop = -1
            ELSE
              dx_out_intrp = x_out_param
              nx_out = int((x_out_high - x_out_low)/dx_out_intrp + 1.1)
              allocate ( x_out(nx_out), y_out(nx_out) )
              DO j = 1, nx_out
                x_out(j) = float(j-1) * dx_out_intrp + x_out_low
              END DO
              dx_out = dx_out_intrp
              m_start = -1
              m_stop = -1
            END IF
        
          END IF
        
        END IF   

        CALL Smooth_interpolate (x_in, y_in, nx_in, 
     :                         x_out, y_out, nx_out, 
     :                         itype, ipow, dx_smooth, smooth_param,
     :                         x_out_param, m_start, m_stop)


! write output

        f_out = ' '
        f_out = f_in(1:nc_f_in)//string4name(1:nc_string4name)
        call trim_c(f_out,nc_f_out)
        call get_lun(nu_out)
        open(unit=nu_out,file=f_out(1:nc_f_out),status='unknown')
  
        write(nu_out,'(a)') ' Input file: '//f_in(1:nc_f_in)
         
        n4out = max(nx_in, nx_out)
          
        allocate ( xin_c(n4out), yin_c(n4out), 
     :             x_out_c(n4out), y_out_c(n4out))
     
        xin_c = ' '
        yin_c = ' '
        x_out_c = ' '
        y_out_c = ' '
        
        do i = 1, nx_in
          write(xin_c(i),'(es11.4)') x_in(i)
          write(yin_c(i),'(es11.4)') y_in(i)
        end do
  
        do i = 1, nx_out
          write(x_out_c(i),'(es11.4)') x_out(i)
          write(y_out_c(i),'(es11.4)') y_out(i)
        end do

        write(nu_out,'(8x,a, 8x,a, 7x,a, 7x,a)') 
     :  'x_in', 'y_in', 'x_out', 'y_out'
     
        do i = 1, n4out
          write(nu_out,'(1x,a11, 1x,a11, 1x,a11, 1x,a11)') 
     :             xin_c(i), yin_c(i), x_out_c(i), y_out_c(i)
        end do
      
        deallocate (y_out) 
        IF (.not. x_out_allocated_in_pp) deallocate (x_out) 
      
        deallocate (xin_c, yin_c, x_out_c, y_out_c) 
        
        close(nu_out)


      END DO loop_over_files

        time_stop = ' '
* Standard Fortran 90 intrinsic Subroutine DATE_AND_TIME
        call DATE_AND_TIME( date, time_stop )
* Date is returned as 'CCYYMMDD'
* Time is returned as 'hhmmss.sss'
        write(nu_sum, '(2a)') '  Time finished processing record: '//
     :           time_stop(1:2)//':'//
     :           time_stop(3:4)//':'//time_stop(5:10)

        call time_diff(time_start, time_stop, time_elapsed)
        write(nu_sum, '(2x,a,1x,1p, e10.3)') 
     :       ' Elapsed time (sec): ', time_elapsed
        write(nu_sum,*)
        write(nu_sum,*)

* Close control and output files:

      close(nu_ctl)
      close(nu_sum)

      stop
      end
* ------------------------------------------------------------------- Smooth_x_y_data


      include '\forprogs\smooth_interpolate.for'
      include '\forprogs\skip.for'
      include '\forprogs\skipcmnt.for'
      include '\forprogs\get_lun.for'
      include '\forprogs\upstr.for'
      include '\forprogs\smc_npts.for'
      include '\forprogs\smcread.for'
      include '\forprogs\imnmax.for'
      include '\forprogs\trim_c.for'
      include '\forprogs\rc_subs.for'
      include '\forprogs\locate.for'
      include '\forprogs\yintrf.for'
      include '\forprogs\fs2write.for'
      include '\forprogs\get_date.for'
      include '\forprogs\get_time.for'
      include '\forprogs\time_diff.for'

 